"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""

from enum import Enum


class Status(Enum):
    EXPIRED = -1
    FULL = -2
    NOTFOUND = -3



# Add some tests here

import unittest
import elara
import time


class RunTests(unittest.TestCase):
    def setUp(self):
        cache_param = {"max_age": 2, "max_size": 2}
        self.db = elara.exe_cache("test.db", cache_param)

    def test_exe_cache(self):
        cache_param = {"max_age": -1, "max_size": 2}
        with self.assertRaises(Exception):
            new_db = elara.exe_cache("new.db", cache_param)

    def test_age(self):
        self.db.set("key1", 1)
        self.assertEqual(self.db.get("key1"), 1)
        time.sleep(2)
        self.assertEqual(self.db.get("key1"), None)

        self.db.set("key2", 2)
        self.assertEqual(self.db.get("key2"), 2)
        time.sleep(1)
        self.assertEqual(self.db.get("key2"), 2)
        time.sleep(1)
        self.assertEqual(self.db.get("key1"), None)

    def test_size(self):
        self.db.set("key1", 1, "i")
        self.db.set("key2", 2, "i")

        time.sleep(2)
        self.assertEqual(self.db.get("key2"), 2)
        self.db.set("key3", 3)
        time.sleep(2)

        self.assertEqual(self.db.get("key3"), None)
        self.assertEqual(self.db.get("key1"), None)
        self.assertEqual(self.db.get("key2"), 2)

    def test_getkeys(self):
        self.db.set("key1", 1, "i")
        self.db.set("key2", 2, "i")
        self.db.set("key3", 3)
        self.assertEqual(self.db.getkeys(), ["key3", "key2"])
        time.sleep(2)
        self.assertEqual(self.db.getkeys(), ["key2"])

    def test_cache_defaults(self):
        db_new = elara.exe_cache("cache.db", {"max_size": 4})
        db_new.set("key1", 1)
        db_new.set("key2", 2)
        db_new.set("key3", 3)
        db_new.set("key4", 4)

        assert db_new.getkeys() == ["key4", "key3", "key2", "key1"]
        db_new.set("key5", 5, 2)
        assert db_new.getkeys() == ["key5", "key4", "key3", "key2"]

        time.sleep(2)
        assert db_new.getkeys() == ["key4", "key3", "key2"]

        db_new.clear()

        db_new = elara.exe_cache("cache2.db", {"max_size": 4, "cull_freq": 50})

        db_new.set("key1", 1)
        db_new.set("key2", 2)
        db_new.set("key3", 3)
        db_new.set("key4", 4)

        db_new.set("key5", 5)

        assert db_new.getkeys() == ["key5", "key4", "key3"]

    def test_overwrite(self):
        cache_param = {"max_age": 900, "max_size": 4, "cull_freq": 25}

        cache = elara.exe_cache("new.db", cache_param)

        cache.set("key1", "This one will be evicted in 900 seconds")
        cache.set(
            "key2", "This one will not be evicted", "i"
        )  # 'i' signifies it will never be evicted
        cache.set("key3", "This one will be evicted in 100 seconds", 2)

        assert cache.getkeys() == ["key3", "key2", "key1"]
        time.sleep(2)
        assert cache.getkeys() == ["key2", "key1"]

        cache.set("key3", 5)
        cache.set("key4", 1)

        assert cache.getkeys() == ["key4", "key3", "key2", "key1"]

        cache.set("key1", 7, "i")  # overwrite "key1" to never expire

        assert cache.getkeys() == ["key1", "key4", "key3", "key2"]
        assert cache.get("key1") == 7

        cache.set("key5", 20)  # Automatic culling when max_size is reached

        assert cache.getkeys() == ["key5", "key1", "key4", "key3"]



# Add tests here

import unittest
from elara import LRU, Cache_obj


class RunTests(unittest.TestCase):
    def setUp(self):
        self.lru = LRU()
        self.tiny_lru = LRU(2)  # Set max_size of the cache
        self.obj1 = Cache_obj("key1")
        self.obj2 = Cache_obj("key2")
        self.lru.push(self.obj1)
        self.lru.push(self.obj2)

    def test_push(self):
        deleted_keys, cache = self.lru.all()
        self.assertEqual(len(deleted_keys), 0)
        x = []
        # append in reverse order to match cache lru
        x.append(self.obj2)
        x.append(self.obj1)
        self.assertEqual(cache, x)

    def test_peek(self):
        self.assertEqual(self.lru.peek(), self.obj2)

    def test_touch(self):
        self.lru.touch(self.obj1.key)
        self.assertEqual(self.lru.peek(), self.obj1)

    def test_clear(self):
        self.lru.clear()
        deleted_keys, cache = self.lru.all()
        self.assertEqual(len(deleted_keys), 0)
        self.assertEqual(len(cache), 0)

    def test_rem(self):
        self.lru.rem(self.obj2)
        deleted_keys, cache = self.lru.all()
        self.assertEqual(cache[0], self.obj1)

    def test_pop(self):
        self.assertEqual(self.lru.pop(), self.obj1)
        deleted_keys, cache = self.lru.all()
        self.assertEqual(len(deleted_keys), 0)
        self.assertEqual(len(cache), 1)



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""

from .lru import Cache_obj
from .status import Status
from .exceptions import InvalidCacheParams


def is_pos(val):
    return isinstance(val, int) and val > 0


# Hash table operations


def hnew(self, key, max_age=None):
    if isinstance(key, str):
        if max_age == None:
            cache_obj = Cache_obj(key, self.max_age)
        else:
            if is_pos(max_age):
                cache_obj = Cache_obj(key, max_age)
            else:
                raise InvalidCacheParams("max_age")

        if self.lru.push(cache_obj) == Status.FULL:
            self.cull(self.cull_freq)  # Automatic cull
            self.lru.push(cache_obj)

        self.db[key] = {}
        self._autocommit()
        return True
    else:
        raise Exception


def hadd(self, key, dict_key, value):
    if self.exists(key):
        self.db[key][dict_key] = value
        self._autocommit()
        return True
    else:
        return False


def haddt(self, key, tuple):
    if self.exists(key):
        self.db[key][tuple[0]] = tuple[1]
        self._autocommit()
        return True
    else:
        return False


def hget(self, key, dict_key):
    if self.exists(key):
        return self.db[key][dict_key]
    else:
        return False


def hpop(self, key, dict_key):
    if self.exists(key):
        value = self.db[key][dict_key]
        del self.db[key][dict_key]
        self._autocommit()
        return value
    else:
        return False


def hkeys(self, key):
    if self.exists(key):
        return self.db[key].keys()
    else:
        return None


def hvals(self, key):
    if self.exists(key):
        return self.db[key].values()
    else:
        return None


def hexists(self, key, dict_key):
    if self.exists(key):
        if dict_key in self.db[key]:
            return True
        else:
            return False
    else:
        return False


def hmerge(self, key, new_dict):
    if self.exists(key):
        first = self.db[key]
        first.update(new_dict)
        self._autocommit()
        return True
    else:
        return False
